#ifndef LOGGER_HPP
#define LOGGER_HPP

#include <boost/log/attributes/clock.hpp>
#include <boost/log/attributes/constant.hpp>
#include <boost/log/attributes/current_thread_id.hpp>
#include <boost/log/filters/attr.hpp>
#include <boost/log/filters/has_attr.hpp>
#include <boost/log/formatters/attr.hpp>
#include <boost/log/formatters/date_time.hpp>
#include <boost/log/formatters/if.hpp>
#include <boost/log/formatters/message.hpp>
#include <boost/log/formatters/stream.hpp>
#include <boost/log/sinks/ordering_async_frontend.hpp>
#include <boost/log/sinks/text_ostream_backend.hpp>
#include <boost/log/sources/record_ostream.hpp>
#include <boost/log/sources/severity_logger.hpp>
#include <boost/log/utility/init/common_attributes.hpp>
#include <boost/log/utility/record_ordering.hpp>

#include <boost/date_time/posix_time/posix_time.hpp>

#include <iostream>
#include <fstream>

//TODO Add a way to build a project with "LOG" command whitout the logger
//TODO Make it possible to change the log filename with a #define whitout the need to recompile
//TODO Make it possible to change the log filename while running
//TODO Make it possible to change the filter level for the console output 

enum severityLevel{unknown, trace, debug, info, warning, error, fatal};

class logger{
private:
	logger();
	logger(const logger&);
	logger& operator=(const logger&);
	
	typedef boost::log::sinks::ordering_asynchronous_sink<boost::log::sinks::text_ostream_backend> textSink;
	typedef boost::log::sources::severity_logger_mt<severityLevel> severity_logger;
	
	boost::shared_ptr<textSink> fileSink;
	boost::shared_ptr<textSink> consoleSink;

	severity_logger innerLogger;
		
	static logger instance;
public:	

	static severity_logger& get(){
		return instance.innerLogger;
	}

	static void setThreadName(const std::string& name){
		boost::log::core::get()->add_thread_attribute("ThreadName", boost::make_shared< boost::log::attributes::constant<std::string> >(name));
	}
	
	~logger();
};

#endif //LOGGER_HPP
